package Chart::Clicker::Decoration::LegendItem;
use strict;

use base 'Chart::Clicker::Decoration::Base';
__PACKAGE__->mk_accessors(qw(color insets label font));

use Chart::Clicker::Drawing::Font;

=head1 NAME

Chart::Clicker::Decoration::LegendItem

=head1 DESCRIPTION

Chart::Clicker::Decoration::LegendItem represents a single item in a legend.

=head1 SYNOPSIS

=head1 METHODS

=head2 Constructor

=over 4

=item new()

Creates a new LegendItem object.

=back

=head2 Class Methods

=over 4

=back

=head1 AUTHOR

Cory 'G' Watson <gphat@onemogin.com>

=head1 SEE ALSO

perl(1)

=cut
1;
